@@ -15,7 +15,6 @@
 #include <stdlib.h>
 
 #include <gdk/gdkx.h>
-#include <gtk/gtkmenu.h>
 #include <gtk/gtksignal.h>
 
 #include <bonobo/bonobo-main.h>
@@ -43,6 +42,7 @@ struct _BonoboControlPrivate {
 	Bonobo_PropertyBag   propbag;
 	BonoboUIContainer   *popup_ui_container;
 	BonoboUIComponent   *popup_ui_component;
+	BonoboUIEngine      *popup_ui_engine;
 	BonoboUISync        *popup_ui_sync;
 
 	GtkWidget           *plug;
@@ -61,6 +61,7 @@ control_frame_connection_died_cb (gpointer connection,
 	g_return_if_fail (control != NULL);
 
 	dprintf ("The remote control frame died unexpectedly");
+	bonobo_object_unref (BONOBO_OBJECT (control));
 }
 
 void
@@ -88,45 +89,6 @@ bonobo_control_add_listener (CORBA_Object        object,
 	}
 }
 
-/* Control lifecycle grind ... */
-void
-bonobo_control_notify_plug_died (BonoboControl *control,
-				 gboolean       inproc_parent_died)
-{
-	Bonobo_ControlFrame frame;
-	gboolean            end_of_life;
-
-	frame = control->priv->frame;
-
-	dprintf ("bonobo_control_notify_plug_died (%d): ",
-		 inproc_parent_died);
-
-	if (frame != CORBA_OBJECT_NIL) {
-		end_of_life = TRUE; /* Hack for now */
-		switch (ORBit_small_get_connection_status (frame)) {
-		case ORBIT_CONNECTION_CONNECTED:
-		case ORBIT_CONNECTION_CONNECTING:
-			dprintf ("valid connection\n");
-			g_warning ("FIXME: We need to wait for 'broken' on the "
-				   "connection - to allow re-parenting");
-			break;
-		case ORBIT_CONNECTION_DISCONNECTED:
-			dprintf ("connection broken\n");
-			break;
-		case ORBIT_CONNECTION_IN_PROC:
-			end_of_life = inproc_parent_died;
-			break;
-		}
-	} else {
-		dprintf ("no frame\n");
-		end_of_life = TRUE;
-	}
-
-	if (end_of_life)
-		bonobo_object_unref (BONOBO_OBJECT (control));
-}
-
-
 /**
  * bonobo_control_window_id_from_x11:
  * @x11_id: the x11 window id.
@@ -584,11 +546,11 @@ bonobo_control_get_automerge (BonoboControl *control)
 }
 
 static void
-bonobo_control_dispose (GObject *object)
+bonobo_control_destroy (BonoboObject *object)
 {
 	BonoboControl *control = (BonoboControl *) object;
 
-	dprintf ("bonobo_control_dispose %p\n", object);
+	dprintf ("bonobo_control_destroy %p\n", object);
 
 	if (control->priv->plug)
 		bonobo_control_set_plug (control, NULL);
@@ -599,16 +561,22 @@ bonobo_control_dispose (GObject *object)
 
 	if (control->priv->widget)
 		g_object_unref (G_OBJECT (control->priv->widget));
+	control->priv->widget = NULL;
 
 	control->priv->popup_ui_container = bonobo_object_unref (
-		BONOBO_OBJECT (control->priv->popup_ui_container));
+		(BonoboObject *) control->priv->popup_ui_container);
+
+	if (control->priv->popup_ui_engine)
+		g_object_unref (G_OBJECT (control->priv->popup_ui_engine));
+	control->priv->popup_ui_engine = NULL;
 
 	control->priv->popup_ui_component = bonobo_object_unref (
-		BONOBO_OBJECT (control->priv->popup_ui_component));
+		(BonoboObject *) control->priv->popup_ui_component);
 
 	control->priv->popup_ui_sync = NULL;
+	control->priv->inproc_frame  = NULL;
 
-	bonobo_control_parent_class->dispose (object);
+	BONOBO_OBJECT_CLASS (bonobo_control_parent_class)->destroy (object);
 }
 
 static void
@@ -850,6 +818,7 @@ static void
 bonobo_control_class_init (BonoboControlClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *)klass;
+	BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)klass;
 	POA_Bonobo_Control__epv *epv;
 
 	bonobo_control_parent_class = g_type_class_peek_parent (klass);
@@ -873,8 +842,8 @@ bonobo_control_class_init (BonoboControlClass *klass)
 			      G_TYPE_NONE, 1,
 			      G_TYPE_BOOLEAN);
 
-	object_class->dispose  = bonobo_control_dispose;
 	object_class->finalize = bonobo_control_finalize;
+	bonobo_object_class->destroy = bonobo_control_destroy;
 
 	epv = &klass->epv;
 
@@ -1185,7 +1154,6 @@ bonobo_control_set_plug (BonoboControl *control,
 		bonobo_plug_set_control (old_plug, NULL);
 		g_object_unref (G_OBJECT (old_plug));
 	}
-
 }
 
 BonoboPlug *
@@ -1215,25 +1183,20 @@ bonobo_control_get_popup_ui_container (BonoboControl *control)
 	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
 
 	if (!control->priv->popup_ui_container) {
-		BonoboUIEngine *engine;
-
-		engine = bonobo_ui_engine_new (G_OBJECT (control));
+		control->priv->popup_ui_engine = bonobo_ui_engine_new (
+			G_OBJECT (control));
 
 		control->priv->popup_ui_sync = bonobo_ui_sync_menu_new (
-			engine, NULL, NULL, NULL);
+			control->priv->popup_ui_engine, NULL, NULL, NULL);
 
 		bonobo_ui_engine_add_sync (
-			engine, control->priv->popup_ui_sync);
+			control->priv->popup_ui_engine,
+			control->priv->popup_ui_sync);
 
 		control->priv->popup_ui_container = bonobo_ui_container_new ();
 		bonobo_ui_container_set_engine (
-			control->priv->popup_ui_container, engine);
-
-		g_signal_connect_closure (
-			G_OBJECT (control->priv->popup_ui_container),
-			"destroy",
-			g_cclosure_new_swap (
-				G_CALLBACK (g_object_unref), engine, NULL), 0);
+			control->priv->popup_ui_container,
+			control->priv->popup_ui_engine);
 	}
 
 	return control->priv->popup_ui_container;
@@ -1261,9 +1224,13 @@ bonobo_control_get_popup_ui_component (BonoboControl *control)
 }
 
 gboolean
-bonobo_control_do_popup (BonoboControl *control,
-			 guint          button,
-			 guint32        activate_time)
+bonobo_control_do_popup_full (BonoboControl       *control,
+			      GtkWidget           *parent_menu_shell,
+			      GtkWidget           *parent_menu_item,
+			      GtkMenuPositionFunc  func,
+			      gpointer             data,
+			      guint                button,
+			      guint32              activate_time)
 {
 	char      *path;
 	GtkWidget *menu;
@@ -1285,8 +1252,19 @@ bonobo_control_do_popup (BonoboControl *control,
 
 	gtk_widget_show (menu);
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-			NULL, NULL, button, activate_time);
+	gtk_menu_popup (GTK_MENU (menu),
+			parent_menu_shell, parent_menu_item,
+			func, data,
+			button, activate_time);
 
 	return TRUE;
 }
+
+gboolean
+bonobo_control_do_popup (BonoboControl       *control,
+			 guint                button,
+			 guint32              activate_time)
+{
+	bonobo_control_do_popup_full (control, NULL, NULL, NULL, NULL,
+				      button, activate_time);
+}
