@@ -400,7 +400,7 @@ bonobo_ui_config_widget_class_init (BonoboUIConfigWidgetClass *klass)
 GtkType
 bonobo_ui_config_widget_get_type (void)
 {
-	static guint bonobo_ui_config_widget_type = 0;
+	static GtkType bonobo_ui_config_widget_type = 0;
 
 	if (!bonobo_ui_config_widget_type) {
 		GtkTypeInfo bonobo_ui_config_widget_info = {
