@@ -18,6 +18,7 @@
 #include <bonobo/bonobo-ui-xml.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-ui-container.h>
+#include <bonobo/bonobo-ui-private.h>
 #include <bonobo/bonobo-ui-engine.h>
 #include <bonobo/bonobo-ui-engine-config.h>
 #include <bonobo/bonobo-ui-engine-private.h>
@@ -65,8 +66,6 @@ struct _BonoboUIEnginePrivate {
 	BonoboUIEngineConfig *config;
 
 	GHashTable   *cmd_to_node;
-
-	guint         destroy_id;
 };
 
 /*
@@ -321,6 +320,7 @@ info_free_fn (BonoboUIXmlData *data)
 	NodeInfo *info = (NodeInfo *) data;
 
 	if (info->object != CORBA_OBJECT_NIL) {
+		dprintf ("** Releasing object %p on info %p\n", info->object, info);
 		bonobo_object_release_unref (info->object, NULL);
 		info->object = CORBA_OBJECT_NIL;
 	}
@@ -732,13 +732,6 @@ sub_component_cmp_name (BonoboUIEngine *engine, const char *name)
 static void
 sub_component_destroy (BonoboUIEngine *engine, SubComponent *component)
 {
-	if (engine->priv->destroy_id)
-		g_signal_handler_disconnect (G_OBJECT (engine->priv->container),
-					     engine->priv->destroy_id);
-	engine->priv->destroy_id = 0;
-
-	engine->priv->container = NULL;
-
 	engine->priv->components = g_slist_remove (
 		engine->priv->components, component);
 
@@ -1127,11 +1120,9 @@ bonobo_ui_engine_object_set (BonoboUIEngine   *engine,
 			gtk_widget_destroy (info->widget);
 		info->widget = NULL;
 	}
-
-	if (object != CORBA_OBJECT_NIL)
-		info->object = bonobo_object_dup_ref (object, ev);
-	else
-		info->object = CORBA_OBJECT_NIL;
+	
+	dprintf ("** Setting object %p on info %p\n", object, info);
+	info->object = bonobo_object_dup_ref (object, ev);
 
 	bonobo_ui_xml_set_dirty (engine->priv->tree, node);
 
@@ -1143,7 +1134,6 @@ bonobo_ui_engine_object_set (BonoboUIEngine   *engine,
 /*	bonobo_ui_engine_dump (win, "After object set updatew");*/
 
 	return BONOBO_UI_ERROR_OK;
-	
 }
 
 /**
@@ -1314,14 +1304,6 @@ bonobo_ui_engine_xml_rm (BonoboUIEngine *engine,
 	return err;
 }
 
-static void
-blank_container (BonoboUIContainer *container,
-		 BonoboUIEngine    *engine)
-{
-	if (engine->priv)
-		engine->priv->container = NULL;
-}
-
 /**
  * bonobo_ui_engine_set_ui_container:
  * @engine: the engine
@@ -1333,17 +1315,30 @@ void
 bonobo_ui_engine_set_ui_container (BonoboUIEngine    *engine,
 				   BonoboUIContainer *ui_container)
 {
+	BonoboUIContainer *old_container;
+
 	g_return_if_fail (BONOBO_IS_UI_ENGINE (engine));
-	g_return_if_fail (engine->priv->container == NULL);
-	g_return_if_fail (BONOBO_IS_UI_CONTAINER (ui_container));
 
-	engine->priv->container = ui_container;
+	if (engine->priv->container == ui_container)
+		return;
 
-	if (ui_container)
-		engine->priv->destroy_id = g_signal_connect_data (
-			G_OBJECT (ui_container), "destroy",
-			G_CALLBACK (blank_container), engine,
-			NULL, 0);
+	g_return_if_fail (!ui_container ||
+			  BONOBO_IS_UI_CONTAINER (ui_container));
+
+	old_container = engine->priv->container;
+
+	if (ui_container) {
+		engine->priv->container = BONOBO_UI_CONTAINER (
+			bonobo_object_ref (BONOBO_OBJECT (ui_container)));
+
+		bonobo_ui_container_set_engine (ui_container, engine);
+	} else
+		engine->priv->container = NULL;
+
+	if (old_container) {
+		bonobo_ui_container_set_engine (old_container, NULL);
+		bonobo_object_unref (BONOBO_OBJECT (old_container));
+	}
 }
 
 /**
@@ -1669,6 +1664,18 @@ impl_emit_event_on (BonoboUIEngine *engine,
 	g_free (component_id);
 }
 
+static void
+impl_dispose (GObject *object)
+{
+	BonoboUIEngine *engine;
+
+	engine = BONOBO_UI_ENGINE (object);
+
+	bonobo_ui_engine_set_ui_container (engine, NULL);
+	
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 impl_finalize (GObject *object)
 {
@@ -1676,6 +1683,8 @@ impl_finalize (GObject *object)
 	BonoboUIEnginePrivate *priv;
 	GSList *l;
 
+	dprintf ("bonobo_ui_engine_finalize %p\n", object);
+       
 	engine = BONOBO_UI_ENGINE (object);
 	priv = engine->priv;
 
@@ -1719,6 +1728,7 @@ class_init (BonoboUIEngineClass *engine_class)
  	sensitive_id = g_quark_from_static_string ("sensitive");
 
 	object_class = G_OBJECT_CLASS (engine_class);
+	object_class->dispose  = impl_dispose;
 	object_class->finalize = impl_finalize;
 
 	engine_class->emit_verb_on  = impl_emit_verb_on;
@@ -2435,9 +2445,8 @@ bonobo_ui_engine_build_control (BonoboUIEngine *engine,
 
 	} else if (info->object != CORBA_OBJECT_NIL) {
 
-		control = bonobo_widget_new_control_from_objref
-			(bonobo_object_dup_ref (info->object, NULL),
-			 CORBA_OBJECT_NIL);
+		control = bonobo_widget_new_control_from_objref (
+			info->object, CORBA_OBJECT_NIL);
 		g_return_val_if_fail (control != NULL, NULL);
 		
 		info->type |= CUSTOM_WIDGET;
